<?php

namespace Drupal\event_scheduler\EventSubscriber;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\State\StateInterface;
use Drupal\event_scheduler\Event\EventScheduleInterface;
use Drupal\event_scheduler\EventSchedulerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\event_scheduler\EventSchedulerDispatcher;
use Drupal\event_scheduler\EventSchedulerDatabaseInterface;
use Symfony\Component\HttpKernel\Event\KernelEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Contracts\EventDispatcher\Event as EventInterface;

/**
 * Class EventsProcessSubscriber
 *
 * @package Drupal\event_scheduler\EventSubscriber
 */
class EventsProcessSubscriber implements EventSubscriberInterface {

  protected const LOCK = 'event-scheduler-process-lock';

  protected EventSchedulerDispatcher $scheduler;

  protected EventSchedulerDatabaseInterface $schedulerDatabase;

  protected TimeInterface $time;

  protected StateInterface $state;

  protected EventSchedulerInterface $eventScheduler;

  protected LoggerChannelInterface $logger;

  /**
   * Constructs a new EventsProcessSubscriber object.
   *
   * @param EventSchedulerDispatcher $scheduler
   * @param EventSchedulerInterface $eventScheduler
   * @param EventSchedulerDatabaseInterface $schedulerDatabase
   * @param TimeInterface $time
   * @param StateInterface $state
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   */
  public function __construct(EventSchedulerDispatcher $scheduler,
                              EventSchedulerInterface $eventScheduler,
                              EventSchedulerDatabaseInterface $schedulerDatabase,
                              TimeInterface $time,
                              StateInterface $state,
                              LoggerChannelInterface $logger
  ) {
    $this->scheduler = $scheduler;
    $this->eventScheduler = $eventScheduler;
    $this->schedulerDatabase = $schedulerDatabase;
    $this->time = $time;
    $this->state = $state;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  static function getSubscribedEvents() {
    // Make sure this happens before the processing of local events.
    $events[KernelEvents::TERMINATE] = ['onKernelTerminate', 1100];

    return $events;
  }

  /**
   * This method is called whenever the KERNEL::TERMINATE event is
   * dispatched.
   *
   * @param KernelEvent $event
   */
  public function onKernelTerminate($event) {
    if ($this->state->get(static::LOCK, FALSE)) {
      // If this is locked then it's already being processed.
      return;
    }

    // Okay, let's do this thing. Prevent other processes from having a look in.
    $this->state->delete(static::LOCK);
    $this->state->set(static::LOCK, TRUE);

    // Time now...
    $now = $this->time->getCurrentTime();

    // Fetch the timestamp of the next scheduled event, if this is
    // zero it means there aren't any, and if the timestamp is in
    // the future we don't want to do anything right now.
    $timestamp = $this->schedulerDatabase->nextScheduledEventTimestamp();
    if (!$timestamp || $timestamp > $now) {
      // Make sure to remove the lock...
      $this->state->delete(static::LOCK);
      return;
    }

    // Find all scheduled events that have passed their
    // deadline but are not already being processed.
    $conditions = [
      'launch' => ['value' => $now, 'op' => '<'],
      'processed' => 0,
    ];

    /** @var EventScheduleInterface | EventInterface $scheduledEvent */
    foreach ($this->eventScheduler->loadEvent($conditions) as $scheduledEvent) {
      // Extract the event's name so we can dispatch it...
      $eventName = $scheduledEvent->getName();
      // And dispatch it normally.
      $this->scheduler->dispatch($scheduledEvent, $eventName);
    }

    // Now delete them.
    $this->schedulerDatabase->delete($conditions);

    // And remove the lock.
    $this->state->delete(static::LOCK);
  }

}
