<?php

namespace Drupal\event_scheduler\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Queue\QueueInterface;
use Drupal\Core\Queue\QueueWorkerInterface;
use Drupal\Core\Queue\SuspendQueueException;
use Drupal\event_scheduler\EventSchedulerDispatcher;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ScheduledEventsForm
 *
 * @package Drupal\event_scheduler\Form
 */
class ScheduledEventsForm extends FormBase {

  use LoggerChannelTrait;

  protected QueueInterface $queue;

  protected QueueWorkerInterface $queueWorker;

  /**
   * Constructs a new ScheduledEventsForm object.
   *
   * @param QueueInterface $queue
   * @param QueueWorkerInterface $queueWorker
   */
  public function __construct(
    QueueInterface $queue,
    QueueWorkerInterface $queueWorker
  ) {
    $this->queue = $queue;
    $this->queueWorker = $queueWorker;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('queue')
        ->get(EventSchedulerDispatcher::QUEUE_NAME),
      $container->get('plugin.manager.queue_worker')
        ->createInstance(EventSchedulerDispatcher::QUEUE_NAME)
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'scheduled_events_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['help'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Submitting this form will process the Manual Queue which contains @number items.',
        ['@number' => $this->queue->numberOfItems()]),
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Process queue'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $logger = $this->getLogger('event_scheduler.manual');

    while ($item = $this->queue->claimItem()) {
      try {
        $this->queueWorker->processItem($item->data);
        $this->queue->deleteItem($item);
      }
      catch (SuspendQueueException $e) {
        $logger->debug($this->t('Failed to process queue item: %x', ['%x' => $e->getMessage()]));
        $this->queue->releaseItem($item);
        break;
      }
      catch (Exception $e) {
        $logger->debug($this->t('Failed to process queue item: %x', ['%x' => $e->getMessage()]));
      }
    }
  }

}
